////////////////////////////////////////////////////////////////////////////////
///
///		Title		: NPhoto Synchronized Object
///
///		Copyright (c) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: Desktop Service Development
///
////////////////////////////////////////////////////////////////////////////////
#pragma once
/**
* @group 	PCM Base Lib	
* @brief 	sync object interface
* @date		08/21/2006
* @author	telltale
*/
interface INPSynchObject
{
public:
	/**
	* @brief 	{kr} ȣ thread NPSynchObject ownership  Ѵ. {/kr}
	* @param	dwMilliseconds	spin count
	* @return	success
	*/
	virtual BOOL Lock(IN DWORD dwMilliseconds = INFINITE) = 0;

	/**
	* @brief 	{kr} ȣ thread  NPSynchObject ownership releases Ѵ. {/kr}
	* @return	success
	*/
	virtual BOOL Unlock() = 0;
};
/**
* @group 	PCM Base Lib	
* @brief 	critical section 
* @date		08/22/2006
* @author	telltale
* @remark	
*/
class CNPCriticalSection : public INPSynchObject
{
public:
	/**
	* @brief 	defualt constructor
	* @param	dwSpinCount		[in] spin count
	*/
	explicit CNPCriticalSection(DWORD dwSpinCount = 4000);

	/**
	* @brief destructor	
	*/
	virtual ~CNPCriticalSection();

	/**
	* @brief 	{kr} ȣ thread critical section object ownership  Ѵ. {/kr}
	*/
	void Enter();

	/**
	* @brief	{kr} ȣ thread critical section object ownership  õѴ. {/kr}
	* @return					
		- TRUE: ȣ thread critical section object ownership ȹ.
			Leave Ǵ Unlock ȣϿ ȹ ownership releases ־ Ѵ.
		- FALSE: ȣ thread critical section object ownership ȹ  
	*/	
	BOOL TryEnter();

	/**
	* @brief 	{kr} ȣ thread  critical section object ownership releases Ѵ. {/kr}
	*/
	void Leave();

	/**
	* @brief 	{kr}  CRITICAL_SECTION ü pointer ȯѴ. {/kr}
	*/
	operator CRITICAL_SECTION*();

	/**
	* @brief 	{kr}  CRITICAL_SECTION ü ڷ  CRITICAL_SECTION object attach Ѵ. {/kr}
	*/
	void Attach(CRITICAL_SECTION* p);
	
	/**
	* @brief 	{kr}  CRITICAL_SECTION ü ڷ  CRITICAL_SECTION object detach Ѵ. {/kr}
	*/
	CRITICAL_SECTION Detach();

	/**
	* @brief 	{kr} Enter() ȣѴ. {/kr}
	*/
	virtual BOOL Lock(DWORD dwMilliseconds = INFINITE);

	/**
	* @brief 	{kr} Leave() ȣѴ. {/kr}
	*/
	virtual BOOL Unlock();

protected:
	struct Context;
	Context* m_pContext;
};

/**
* @group 	PCM Base Lib	
* @brief 	INPSynchObject scope guarder
* @date		08/22/2006
* @author	telltale
* @remark	Example\n
class CNPXXX\n
{\n
public:\n
	void DoSomething()\n
	{\n
		CNPLock lock(m_csThis);\n
		// Լ   critical section ȴ.\n
	}\n
private:\n
	CNPCriticalSection m_csThis;\n
}\n
*/
class CNPLock
{
public:
	/**
	* @brief 	constructor
	*/
	explicit CNPLock(INPSynchObject& r, DWORD dwMilliseconds = INFINITE);

	/**
	* @brief 	constructor
	*/
	explicit CNPLock(INPSynchObject* p, DWORD dwMilliseconds = INFINITE);

	/**
	* @brief destructor	
	*/
	virtual ~CNPLock();

	/**
	* @brief {kr}  NPSynchObject Lock Ѵ. {/kr} 
	* @param	dwMilliseconds		[in] spin count
	* @return	success
	*/
	BOOL Lock(DWORD dwMilliseconds = INFINITE);

	/**
	* @brief {kr}  NPSynchObject Unlock Ѵ. {/kr} 
	* @return	success
	*/
	BOOL Unlock();

	/**
	* @brief {kr}  NPSynchObject ü Lock  ˻Ѵ. {/kr} 
	* @return	lock
	*/
	BOOL IsLocked();

	/**
	* @brief {kr}  NPSynchObject ü pObj attach Ѵ. {/kr} 
	*/
	void Attach(INPSynchObject* pObj);

	/**
	* @brief {kr}  NPSynchObject ü detach Ѵ. {/kr} 
	* @return	lock
	*/
	INPSynchObject* Detach();

protected:
	struct Context;
	enum { ContextSize = 12 };
	BYTE m_context[ContextSize];
};